@@ -205,7 +205,7 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 ###  if (dep.model==4 & !is.null(cif2) ) dep.model <- 6 ## two cause random cif
 ###  if (dep.model==6) stop("Different causes  under development \n"); 
 
-
+  
   obj <- function(par) 
     { ## {{{
 
@@ -473,7 +473,7 @@ dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
 ##' @author Thomas Scheike
 ##' @references
 ##' Cross odds ratio Modelling of dependence for
-##' Multivariate Competing Risks Data, Scheike and Sun (2010), work in progress.
+##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 
 ##' 
 ##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
 ##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
@@ -589,6 +589,7 @@ cor.cif<-function(cif,data,cause=NULL,times=NULL,
   fit$call <- match.call()
   fit
 } ## }}}
+
 ##' @export
 rr.cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
                  cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
@@ -896,6 +897,7 @@ print.summary.cor <- function(x,digits=3,...)
 ##' models of the type cor.cif, rr.cif or or.cif for the given cumulative incidences and the different dependence
 ##' measures in the object.
 ##'
+##'
 ##' @title Summary for dependence models for competing risks
 ##' @param object object from cor.cif rr.cif or or.cif for dependence between competing risks data for two causes.
 ##' @param marg.cif a number that gives the cumulative incidence in one time point for which concordance and 
@@ -912,10 +914,11 @@ print.summary.cor <- function(x,digits=3,...)
 ##' \item{cif2}{cumulative incidence for cause1.}
 ##' @references
 ##' Cross odds ratio Modelling of dependence for
-##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics to appear.
+##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 
 ##' 
 ##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
 ##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
+##' 
 ##' @author Thomas Scheike
 ##' @keywords survival
 ##' @examples
@@ -1052,7 +1055,24 @@ print.cor<-function(x,digits=3,...)
   print(summary(x)); 
 } ## }}}
 
-##' Concordance
+##' @export
+concordance.cor <- function(object,...) {
+    concordanceCor(object,...)
+}
+
+##' Concordance for Twins
+##'
+##' The concordance is the probability that both twins have experienced the 
+##' event of interest and is defined as 
+##' \deqn{
+##'  cor(t) = P(T_1 \leq t, \epsilon_1 =1 , T_2 \leq t, \epsilon_2=1) 
+##'  }
+##'
+##' Similarly, the casewise concordance is 
+##' \deqn{
+##'  casewise(t) = \frac{cor(t)}{P(T_1 \leq t, \epsilon_1=1) }
+##' }
+##' that is the probability that twin "2" has the event given that twins "1" has. 
 ##'
 ##' @title Concordance Computes concordance and casewise concordance
 ##' @param object Output from the cor.cif, rr.cif or or.cif function
@@ -1063,8 +1083,18 @@ print.cor<-function(x,digits=3,...)
 ##' @param coefs Specfifies  dependence parameters if object is not given.
 ##' @param ...  Extra arguments, not used.
 ##' @author Thomas Scheike
-##' @export
-concordance <- function(object,cif1,cif2=NULL,messages=TRUE,model=NULL,coefs=NULL,...)
+##' @aliases concordanceCor concordance.cor 
+##' @references
+##' Estimating twin concordance for bivariate competing risks twin data
+##' Thomas H. Scheike, Klaus K. Holst and Jacob B. Hjelmborg, 
+##' Statistics in Medicine 2014, 1193-1204
+##' 
+##' Estimating Twin Pair Concordance for Age of Onset.
+##' Thomas H. Scheike, Jacob V B Hjelmborg, Klaus K. Holst, 2015 
+##' in Behavior genetics DOI:10.1007/s10519-015-9729-3
+##' 
+##' @export 
+concordanceCor <- function(object,cif1,cif2=NULL,messages=TRUE,model=NULL,coefs=NULL,...)
 { ## {{{
 
   if (is.null(model)) { 
