@@ -5,6 +5,7 @@
 ##' @param object is phreg object 
 ##' @param n.sim number of simulations for score processes
 ##' @param silent to show timing estimate will be produced for longer jobs
+##' @param robust to control wether robust dM_i(t) or dN_i  are used for simulations
 ##' @param ... Additional arguments to lower level funtions
 ##' @author THomas Scheike and Klaus K. Holst
 ##' @export
@@ -18,9 +19,15 @@
 ##' plot(gg)
 ##' 
 ##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes,data=TRACE) 
+##' ## to get Martingale ~ dN based simulations
 ##' gg <- gof(m1)
+##' 
+##' ## to get Martingale robust simulations, specify cluster in  call 
+##' m1 <- phreg(Surv(time,status==9)~chf+diabetes+cluster(id),data=TRACE) 
+##' gg <- gof(m1)
+##' 
 ##' @export
-gof.phreg  <- function(object,n.sim=1000,silent=1,...)
+gof.phreg  <- function(object,n.sim=1000,silent=1,robust=NULL,...)
 {# {{{
 
 ### test for proportionality 
@@ -39,11 +46,40 @@ sup <- matrix(0,n.sim,nrow(ii))
 hatti <- matrix(0,nd,nrow(ii))
 obs <- apply(abs(Ut),2,max)
 
-tt <- system.time(simcox1<-.Call("PropTestCox",U,Pt,10,obs,PACKAGE="mets"))
-prt <- n.sim*tt[3]/(10*60)
-if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
+if (is.null(robust)) 
+     if (!is.null(object$id)) robust <- TRUE else robust <- FALSE
 
-simcox <-  .Call("PropTestCox",U,Pt,n.sim,obs,PACKAGE="mets")
+### cluster call or robust \hat M_i(t) based  
+if (robust) {
+        xx <- object$cox.prep
+        S0i <- rep(0,length(xx$strata))
+        S0i[xx$jumps+ 1] <- 1/object$S0
+        Z <- xx$X
+        ZdN <- U <- E <- matrix(0, nrow(xx$X), object$p)
+        E[xx$jumps + 1, ] <- object$E
+        U[xx$jumps + 1, ] <- object$U
+        cumhaz <- c(cumsumstrata(S0i, xx$strata, xx$nstrata))
+        EdLam0 <- apply(E * S0i, 2, cumsumstrata, xx$strata, xx$nstrata)
+        rr <- c(xx$sign * exp(Z %*% coef(object) + xx$offset))
+        MGt <- U[, drop = FALSE] - (Z * cumhaz - EdLam0) * rr * c(xx$weights)
+        ### also weights 
+        w <- c(xx$weights)
+        nn <- nrow(Z)
+
+	tt <- system.time(simcox1<- .Call("PropTestCoxClust",MGt,Pt,w*rr,Z,cumhaz,EdLam0,10,obs,nn,xx$id,xx$strata,xx$nstrata,xx$jumps))
+
+	prt <- n.sim*tt[3]/(10*60)
+	if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
+
+	simcox <- .Call("PropTestCoxClust",MGt,Pt,w*rr,Z,cumhaz,EdLam0,n.sim,obs,nn,xx$id,rep(0,nn),1,xx$jumps)
+} else {
+### no  or dN_i based  
+	tt <- system.time(simcox1<-.Call("PropTestCox",U,Pt,10,obs,PACKAGE="mets"))
+	prt <- n.sim*tt[3]/(10*60)
+	if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
+
+	simcox <-  .Call("PropTestCox",U,Pt,n.sim,obs,PACKAGE="mets")
+}
 sup <-  simcox$supUsim
 res <- cbind(obs,simcox$pval)
 colnames(res) <- c("Sup|U(t)|","pval")
@@ -55,7 +91,7 @@ prmatrix(round(res,digits=2))
 }
 
 out <- list(jumptimes=object$jumptimes,supUsim=sup,res=res,supU=obs,
-	    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,type="prop")
+	    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,type="prop",robust=robust)
 class(out) <- "gof.phreg"
 return(out)
 }# }}}
@@ -76,28 +112,36 @@ return(out)
 ##' @author THomas Scheike and Klaus K. Holst
 ##' @export
 ##' @examples
+##' library(mets)
 ##' data(TRACE)
+##' set.seed(1)
+##' TRACEsam <- blocksample(TRACE,idvar="id",replace=FALSE,100)
 ##' 
-##' dcut(TRACE)  <- ~. 
-##' mm <- model.matrix(~-1+factor(wmicat.4),data=TRACE)
-##' m1 <- gofM.phreg(Surv(time,status==9)~vf+chf+wmi,data=TRACE,modelmatrix=mm) 
+##' dcut(TRACEsam)  <- ~. 
+##' mm <- model.matrix(~-1+factor(wmicat.4),data=TRACEsam)
+##' m1 <- gofM.phreg(Surv(time,status==9)~vf+chf+wmi,data=TRACEsam,modelmatrix=mm)
 ##' summary(m1)
+##' if (interactive()) {
 ##' par(mfrow=c(2,2))
 ##' plot(m1)
+##' }
 ##' 
-##' m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACE,modelmatrix=mm) 
+##' m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACEsam,modelmatrix=mm) 
 ##' summary(m1)
 ##' 
 ##' ## cumulative sums in covariates, via design matrix mm 
-##' mm <- cumContr(TRACE$wmi,breaks=10,equi=TRUE)
-##' m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACE,
+##' mm <- cumContr(TRACEsam$wmi,breaks=10,equi=TRUE)
+##' m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACEsam,
 ##' 		  modelmatrix=mm,silent=0)
 ##' summary(m1)
 ##' 
-##' ## cumulative sums in covariates, via design matrix mm 
-##' m1 <- gofZ.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACE,vars="wmi")
-##' summary(m1)
-##' 
+##' ## cumulative sums in covariates, via design matrix mm
+##' \donttest{ ## Reduce Ex.Timings
+##' m1 <- gofZ.phreg(Surv(time,status==9)~strata(vf)+chf+wmi+age,data=TRACEsam,
+##'                  vars=c("wmi","age"))
+##' summary(m1) 
+##' plot(m1,type="z")
+##' }
 ##' @aliases cumContr gofZ.phreg
 ##' @export
 gofM.phreg  <- function(formula,data,offset=NULL,weights=NULL,modelmatrix=NULL,
@@ -107,7 +151,7 @@ gofM.phreg  <- function(formula,data,offset=NULL,weights=NULL,modelmatrix=NULL,
 if (is.null(modelmatrix)) stop(" must give matrix for cumulating residuals\n"); 
 
 cox1 <- phreg(formula,data,offset=NULL,weights=NULL,Z=modelmatrix,cumhaz=FALSE,...) 
-offsets <- as.matrix(cox1$model.frame[,names(cox1$coef)]) %*% cox1$coef
+offsets <- cox1$X %*% cox1$coef
 if (!is.null(offset)) offsets <- offsets*offset
 if (!is.null(cox1$strata)) 
      coxM <- phreg(cox1$model.frame[,1]~modelmatrix+strata(cox1$strata),data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,...)
@@ -120,6 +164,8 @@ U <- coxM$U
 Ubeta <- cox1$U
 ii <- -solve(cox1$hessian)
 EE <- .Call("vecMatMat",coxM$E,cox1$E,PACKAGE="mets")$vXZ; 
+###print(dim(EE))
+###print(cox1$ZX)
 Pt <- cox1$ZX - EE
 Pt <- apply(Pt,2,cumsum)
 betaiid <- t(ii %*% t(Ubeta))
@@ -154,7 +200,34 @@ return(out)
 }# }}}
 
 ##' @export
-gofZ.phreg  <- function(formula,data,vars,offset=NULL,weights=NULL,breaks=10,equi=TRUE,
+gofZ.phreg  <- function(formula,data,vars,offset=NULL,weights=NULL,breaks=20,equi=TRUE,
+			n.sim=1000,silent=1,...)
+{# {{{
+ gres <- list()
+ res <- matrix(0,length(vars),2)
+ colnames(res) <- c("Sup_z |U(tau,z)|","pval")
+ rownames(res) <- vars
+
+i <- 1
+for (vv in vars) {
+ modelmatrix <- cumContr(data[,vv],breaks=breaks,equi=equi)
+ lres <- gofM.phreg(formula,data,modelmatrix=modelmatrix) 
+
+ res[i,] <- c(lres$Utlast,lres$pval.last)
+ i <- i+1
+
+ lres <- list(lres)
+ names(lres) <- vv
+ gres <- c(gres,lres)
+}
+
+out <- list(res=res,Zres=gres,type="Zmodelmatrix")
+class(out) <- "gof.phreg"
+
+return(out)
+}# }}}
+
+gofZ.phregGam  <- function(formula,data,vars,offset=NULL,weights=NULL,breaks=40,equi=TRUE,
 			n.sim=1000,silent=1,...)
 {# {{{
 
@@ -167,7 +240,7 @@ for (vv in vars) {
  modelmatrix <- cumContr(data[,vv],breaks=breaks,equi=equi)
 
 cox1 <- phreg(formula,data,offset=NULL,weights=NULL,Z=modelmatrix,cumhaz=FALSE,...) 
-offsets <- as.matrix(cox1$model.frame[,names(cox1$coef)]) %*% cox1$coef
+offsets <- cox1$X %*% cox1$coe
 if (!is.null(offset)) offsets <- offsets*offset
 
 if (!is.null(cox1$strata)) 
@@ -268,6 +341,7 @@ cumContr <- function(data, breaks = 4, probs = NULL,equi = TRUE,na.rm=TRUE,...)
 ##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACE) 
 ##' m2 <- phreg(Surv(time,status==9)~vf+strata(chf)+wmi,data=TRACE) 
 ##' par(mfrow=c(2,2))
+##' 
 ##' gofG.phreg(m1)
 ##' gofG.phreg(m2)
 ##' 
@@ -330,8 +404,10 @@ for (j in (i+1):(nstrata-1)) {
 }# }}}
 
 ##' @export
-plot.gof.phreg <-  function(x,col=3,...)
+plot.gof.phreg <-  function(x,col=3,type="time",...)
 {# {{{
+
+if (type=="time") {
 p <- ncol(x$score)
 for (i in 1:p)
 {
@@ -343,6 +419,25 @@ title(main=rownames(x$res)[i])
 matlines(x$jumptimes,simU,type="s",lwd=0.3,col=col)
 lines(x$jumptimes,x$score[,i],type="s",lwd=1.5)
 }
+} else {
+	if (x$type=="modelmatrix") {
+		obsz <- c(tail(x$score,1))
+		times <- 1:length(obsz)
+		rsU <- max(max(abs(obsz)),max(abs(x$simUtlast[1:50,])))
+		plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
+		matlines(times,t(x$simUtlast[1:50,]),type="l",lwd=0.3,col=col)
+	} else {
+	   for (i in 1:length(x$Zres))
+	   {
+	    xr <- x$Zres[[i]]
+	    obsz <- c(tail(xr$score,1))
+	    times <- 1:length(obsz)
+	    rsU <- max(max(abs(obsz)),max(abs(xr$simUtlast[1:50,])))
+	    plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
+	    matlines(times,t(xr$simUtlast[1:50,]),type="l",lwd=0.3,col=col)
+           }
+	}
+}
 
 }# }}}
 
