@@ -876,11 +876,14 @@ recurrentMarginalgam <- function(recurrent,death,fixbeta=NULL,km=TRUE,...)
 }# }}}
 
 ##' @export
-recmarg <- function(recurrent,death,km=TRUE,...)
+recmarg <- function(recurrent,death,Xr=NULL,Xd=NULL,km=TRUE,...)
 {# {{{
   xr <- recurrent
   dr <- death 
 
+  if (!is.null(Xr)) rr <- exp(sum(xr$coef * Xr)) else rr <- 1
+  if (!is.null(Xd)) rrd <- exp(sum(dr$coef * Xd)) else rrd <- 1
+
   ### marginal expected events  int_0^t G(s) \lambda_r(s) ds 
   # {{{
   strat <- dr$strata[dr$jumps]
@@ -891,8 +894,8 @@ recmarg <- function(recurrent,death,km=TRUE,...)
   S0i2[xx$jumps+1] <- 1/x$S0^2
   if (!km) { 
      cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
-     St      <- exp(-cumhazD)
-  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
+     St      <- exp(-cumhazD*rrd)
+  } else St <- exp(rrd*c(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
   ###
   x <- xr
   xx <- x$cox.prep
@@ -900,7 +903,7 @@ recmarg <- function(recurrent,death,km=TRUE,...)
   S0i[xx$jumps+1] <-  1/x$S0
   S0i2[xx$jumps+1] <- 1/x$S0^2
   cumhazDR <- cbind(xx$time,cumsumstrata(St*S0i,xx$strata,xx$nstrata))
-  mu <- cumhazDR[,2]
+  mu <- rr*cumhazDR[,2]
 # }}}
 
  varrs <- data.frame(mu=mu,time=xr$time,strata=xr$strata,St=St)
@@ -910,8 +913,8 @@ recmarg <- function(recurrent,death,km=TRUE,...)
  ### making output such that basehazplot can work also
  out <- list(mu=varrs$mu,time=varrs$time,
 	     St=varrs$St,cumhaz=cbind(varrs$time,varrs$mu),
-             strata=varrs$strata,nstrata=xr$nstrata,jumps=1:nrow(varrs),strata.name=xr$strata.name,
-             strata.level=recurrent$strata.level)
+             strata=varrs$strata,nstrata=xr$nstrata,jumps=1:nrow(varrs),
+	     strata.name=xr$strata.name,strata.level=recurrent$strata.level)
  return(out)
 }# }}}
 
@@ -949,17 +952,17 @@ squareintHdM <- function(phreg,ft=NULL,fixbeta=NULL,...)
   mid <- max(id)+1
   ### also weights 
   w <- c(xx$weights)
-  xxx <- w*(ft*S0i-rr*cumS0i2)
+  xxx <- (ft*S0i-rr*cumS0i2)
   ssf <- cumsumidstratasum(xxx,id,mid,xx$strata,xx$nstrata)$sumsquare
   ss <-  revcumsumidstratasum(w*rr,id,mid,xx$strata,xx$nstrata)$lagsumsquare*cumS0i2^2
   covv <- covfridstrata(xxx,w*rr,id,mid,xx$strata,xx$nstrata)$covs*cumS0i2
-  varA1 <- c(ssf+ss+2*covv)
+  varA1 <- c(ssf+ss-2*covv)
 
   vbeta <- betaiidR <- NULL
   if (fixbeta==0) {# {{{
      invhess <- -solve(x$hessian)
      MGt <- U[,drop=FALSE]-(Z*cumhaz[,2]-EdLam0)*rr*c(xx$weights)
-     UU <- apply(MGt,2,sumstrata,id,max(id)+1)
+     UU <- apply(MGt,2,sumstrata,id,mid)
      betaiidR <- UU %*% invhess
      vbeta <- crossprod(betaiidR)
      varbetat <-   rowSums((Ht %*% vbeta)*Ht)
@@ -969,12 +972,14 @@ squareintHdM <- function(phreg,ft=NULL,fixbeta=NULL,...)
      covk1 <- apply(xxx*betakt,2,cumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="sum")
      covk2 <- apply(w*rr*betakt,2,revcumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="lagsum")
      covk2 <- c(covk2)*cumS0i2
+     covv <- covk1-covk2
      ###
-     varA1 <- varA1+varbetat-2*apply((covk1-covk2)*Ht,1,sum)
+     varA1 <- varA1+varbetat-2*apply(covv*Ht,1,sum)
   }# }}}
 
   return(list(xx=xx,Ht=Ht,varInt=varA1,xxx=xxx,rr=rr,
-	      cumS0i2=cumS0i2,mid=mid,id=id,betaiid=betaiidR,vbeta=vbeta))
+	      cumhaz=cumhaz,cumS0i2=cumS0i2,mid=mid,id=id,
+	      betaiid=betaiidR,vbeta=vbeta,covv=covv))
 } # }}}
 
 ##' @export
@@ -1056,7 +1061,7 @@ covIntH1dM1IntH2dM2 <- function(square1,square2,fixbeta=1,mu=NULL)
 
 ##' @export
 tie.breaker <- function(data,stop="time",start="entry",status="status",id=NULL,ddt=NULL,exit.unique=TRUE)
- {# {{{
+{# {{{
 
    if (!is.null(id)) id <- data[,id]
    ord <- 1:nrow(data)
@@ -1165,7 +1170,7 @@ tie.breaker <- function(data,stop="time",start="entry",status="status",id=NULL,d
 ##'  showfitsim(causes=1,rr,dr,base1,base1)
 ##'
 ##' @export
-##' @aliases simRecurrent showfitsim  simRecurrentGamma covIntH1dM1IntH2dM2 recurrentMarginalgam squareintHdM
+##' @aliases simRecurrent showfitsim  simRecurrentGamma covIntH1dM1IntH2dM2 recurrentMarginalgam squareintHdM addCums
 simRecurrent <- function(n,cumhaz,death.cumhaz=NULL,cumhaz2=NULL,
 			    gap.time=FALSE,
 			    max.recurrent=100,dhaz=NULL,haz2=NULL,
@@ -1241,9 +1246,6 @@ simRecurrent <- function(n,cumhaz,death.cumhaz=NULL,cumhaz2=NULL,
   tall <- dtransform(tall,status=0,time>dtime)
   tall <- dtransform(tall,time=dtime,time>dtime)
   tt <- tall
-###  gemsim <- as.data.frame(matrix(0,max.recurrent*n,ncol(tall)))
-###  names(gemsim) <- names(tall)
-###  gemsim[1:n,] <- tall; 
   nrr <- n
   i <- 1; 
   while (any(tt$time<tt$dtime) & i < max.recurrent) {
@@ -1255,13 +1257,9 @@ simRecurrent <- function(n,cumhaz,death.cumhaz=NULL,cumhaz2=NULL,
 	  tt <- dtransform(tt,status=0,time>dtime)
 	  tt <- dtransform(tt,time=dtime,time>dtime)
 	  nt <- nrow(tt)
-###	  rownames(tt) <- c((nrr+1):(nrr+nt))
-###	  gemsim[(nrr+1):(nrr+nt),] <- tt
-###	  print(rownames(tall)); print(rownames(tt))
 	  tall <- rbind(tall,tt,row.names=NULL)
 	  nrr <- nrr+nt
   }
-###  tall <- gemsim[1:nrr,]
   dsort(tall) <- ~id+entry+time
 
   ### cause 2 is there then decide if jump is 1 or 2
@@ -1290,6 +1288,30 @@ simRecurrent <- function(n,cumhaz,death.cumhaz=NULL,cumhaz2=NULL,
   return(tall)
   }# }}}
 
+
+lin.approx <- function(x2,xfx,x=1) {# {{{
+   ### x=1   gives  f(x2) 
+   ### x=-1  gives  f^-1(x2) 
+   breaks <- xfx[,x]
+   fx     <- xfx[,-x]
+   ri <- sindex.prodlim(breaks,x2)
+   rrr <- (x2-breaks[ri])/(breaks[ri+1]-breaks[ri])
+   res <- rrr*(fx[ri+1]-fx[ri])+fx[ri]
+   res[is.na(res)] <- tail(fx,1)
+   return(res)
+}#
+
+##' @export
+addCums <- function(cumB,cumA,max=5)
+{# {{{
+ times <- sort(unique(c(cumB[,1],cumA[,1])))
+ times <- times[times<max]
+ cumBjx <- lin.approx(times,cumB,x=1)
+ cumAjx <- lin.approx(times,cumA,x=1)
+ cumBA <- cumBjx+cumAjx
+ return(cbind(times,cumBA))
+}# }}}
+
 ##' @export
 simRecurrentGamma <- function(n,haz=0.5,death.haz=0.1,haz2=0.1,max.recurrent=100,var.z=2,times=5000) 
 {# {{{
@@ -1395,11 +1417,12 @@ simRecurrentGamma <- function(n,haz=0.5,death.haz=0.1,haz2=0.1,max.recurrent=100
 ##' ### now with two event types and second type has same rate as death rate
 ##' ######################################################################
 ##'
-##' rr <- simRecurrentII(1000,base1,dr,death.cumhaz=base4)
+##' rr <- simRecurrentII(1000,base1,base4,death.cumhaz=dr)
 ##' dtable(rr,~death+status)
 ##' par(mfrow=c(2,2))
 ##' showfitsim(causes=2,rr,dr,base1,base4)
 ##'
+##' @aliases simRecurrentIII 
 ##' @export
 simRecurrentII <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,
 		    gap.time=FALSE,max.recurrent=100,dhaz=NULL,haz2=NULL,
@@ -1424,9 +1447,6 @@ simRecurrentII <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,
 ###	      print(cor(z))
 	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
       } else if (dependence==3) {
-              stdevs <- var.z^.5
-              b <- stdevs %*% t(stdevs)  
-              covv  <- b * cor.mat  
 	      z <- matrix(rgamma(3*n,1),n,3)
               z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
               z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
@@ -1563,6 +1583,145 @@ showfitsim <- function(causes=2,rr,dr,base1,base4)
   }
 }# }}}
 
+##' @export
+simRecurrentIII <- function(n,cumhaz,cumhaz2,beta1,beta2,death.cumhaz=NULL,
+		    gap.time=FALSE,max.recurrent=100,dhaz=NULL,haz2=NULL,
+		    dependence=0,var.z=0.22,cor.mat=NULL,cens=NULL,...) 
+{# {{{
+
+  fdeath <- dtime <- NULL # to avoid R-check 
+
+  if (dependence==0) { z <- z1 <- z2 <- zd <- rep(1,n) # {{{
+     } else if (dependence==1) {
+	      z <- rgamma(n,1/var.z[1])*var.z[1]
+###	      z <- exp(rnorm(n,1)*var.z[1]^.5)
+	      z1 <- z; z2 <- z; zd <- z
+	      if (!is.null(cor.mat)) { zd <- rep(1,n); }
+      } else if (dependence==2) {
+              stdevs <- var.z^.5
+              b <- stdevs %*% t(stdevs)  
+              covv  <- b * cor.mat  
+	      z <- matrix(rnorm(3*n),n,3)
+	      z <- exp(z%*% chol(covv))
+###	      print(summary(z))
+###	      print(cor(z))
+	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
+      } else if (dependence==3) {
+	      z <- matrix(rgamma(3*n,1),n,3)
+              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
+              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
+              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
+	      z <- cbind(z1,z2,zd)
+###	      print(summary(z))
+###	      print(cor(z))
+      } else stop("dependence 0-3"); # }}}
+
+  cumhaz <- rbind(c(0,0),cumhaz)
+  ll <- nrow(cumhaz)
+  max.time <- tail(cumhaz[,1],1)
+
+  ## extend cumulative for cause 2 to full range of cause 1
+  if (!is.null(cumhaz2)) {# {{{
+      cumhaz2 <- rbind(c(0,0),cumhaz2)
+	      if (is.null(haz2)) 
+		      haz2 <- tail(cumhaz2[,2],1)/tail(cumhaz2[,1],1)
+	      ### linear extrapolation of mortality using given dhaz or 
+      if (tail(cumhaz2[,1],1)<max.time) {
+	      cumhaz2 <- rbind(cumhaz2,c(max.time,haz2*max.time)) 
+       }
+  }# }}}
+
+  ## extend cumulative for death to full range  of cause 1
+  if (!is.null(death.cumhaz)) {# {{{
+     cumhazd <- rbind(c(0,0),death.cumhaz)
+     if (tail(death.cumhaz[,1],1)<max.time) {
+	      ### linear extrapolation of mortality using given dhaz or 
+	      if (is.null(dhaz)) 
+                 dhaz <- tail(death.cumhaz[,2],1)/tail(death.cumhaz[,1],1)
+              cumhazd <- rbind(cumhazd,c(max.time,dhaz*max.time)) 
+       }
+  }# }}}
+
+
+### recurrent first time
+  tall1 <- timereg::pc.hazard(cumhaz,z1)
+  tall2 <- timereg::pc.hazard(cumhaz2,z2)
+  tall <- tall1 
+  tall$status <- ifelse(tall1$time<tall2$time,tall1$status,2*tall2$status)
+  tall$time <- ifelse(tall1$time<tall2$time,tall1$time,tall2$time)
+  tall$id <- 1:n
+  tall$rr2 <- tall2$rr
+### death time simulated
+  if (!is.null(death.cumhaz)) {# {{{
+	  timed   <- timereg::pc.hazard(cumhazd,zd)
+	  tall$dtime <- timed$time
+	  tall$fdeath <- timed$status
+	  if (!is.null(cens)) { 
+             ctime <- rexp(n)/cens
+	     tall$fdeath[tall$dtime>ctime] <- 0; 
+	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
+	  }
+  } else { 
+	  tall$dtime <- max.time; 
+	  tall$fdeath <- 0; 
+	  cumhazd <- NULL 
+	  if (!is.null(cens)) { 
+             ctime <- rexp(n)/cens
+	     tall$fdeath[tall$dtime>ctime] <- 0; 
+	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
+	  }
+  }# }}}
+
+### fixing the first time to event
+  tall$death <- 0
+  tall <- dtransform(tall,death=fdeath,time>dtime)
+  tall <- dtransform(tall,status=0,time>dtime)
+  tall <- dtransform(tall,time=dtime,time>dtime)
+  tall <- count.history(tall,lag=FALSE)
+  ddrop(tall) <- ~lbnr__id
+  dtable(tall,~"Count*")
+  tt <- tall
+  ### setting aside memory 
+  tt1 <- tt2 <- tt
+###  gemsim <- as.data.frame(matrix(0,max.recurrent*n,ncol(tall)))
+###  names(gemsim) <- names(tall)
+###  gemsim[1:n,] <- tall; nrr <- n
+  i <- 1; 
+  while (any(tt$time<tt$dtime) & i < max.recurrent) {
+	  i <- i+1
+	  still <- subset(tt,time<dtime)
+	  nn <- nrow(still)
+	  rr1 <- exp(beta1*still$Count1)*z2[still$id]
+	  rr2 <- exp(beta2*still$Count2)*z1[still$id]
+###	  print(dtable(still,~"Count*"))
+          tt1 <- timereg::pc.hazard(cumhaz,rr2,entry=still$time)
+          tt2 <- timereg::pc.hazard(cumhaz2,rr1,entry=still$time)
+	  tt <- tt1
+          tt$status <- ifelse(tt1$time<=tt2$time,tt1$status,2*tt2$status)
+          tt$time <-   ifelse(tt1$time<=tt2$time,tt1$time,tt2$time)
+	  tt$rr2 <- tt2$rr
+           ###
+	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
+	  tt <- dtransform(tt,death=fdeath,time>dtime)
+	  tt <- dtransform(tt,status=0,time>dtime)
+	  tt <- dtransform(tt,time=dtime,time>dtime)
+	  tt$Count1 <- still$Count1+(tt$status==1)
+	  tt$Count2 <- still$Count2+(tt$status==2)
+	  nt <- nrow(tt)
+	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
+  }
+  dsort(tall) <- ~id+entry+time
+  tall$start <- tall$entry
+  tall$stop  <- tall$time
+
+  attr(tall,"death.cumhaz") <- cumhazd
+  attr(tall,"cumhaz") <- cumhaz
+  attr(tall,"cumhaz2") <- cumhaz2
+  attr(tall,"z") <- z
+
+  return(tall)
+  }# }}}
+
 
 ##' Counts the number of previous events of two types for recurrent events processes
 ##'
